@@ -297,4 +297,14 @@ public class ActiveMQComponent extends JmsComponent implements EndpointCompleter
         }
         return answer;
     }
+
+    /**
+     * We don't want to ever auto-wire the connection factory from the spring app context (requires Camel 2.18 onwards)
+     *
+     * @return false
+     */
+    public boolean isAllowAutoWiredConnectionFactory() {
+        return false;
+    }
+
 }
