@@ -16,34 +16,56 @@
  */
 package org.apache.activemq.store.kahadb.disk.journal;
 
-import java.io.*;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.FilenameFilter;
+import java.io.IOException;
+import java.io.RandomAccessFile;
+import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
+import java.nio.channels.ClosedByInterruptException;
 import java.nio.channels.FileChannel;
-import java.util.*;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedList;
+import java.util.Map;
+import java.util.Set;
+import java.util.TreeMap;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.ScheduledFuture;
+import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.zip.Adler32;
 import java.util.zip.Checksum;
+
 import org.apache.activemq.store.kahadb.disk.util.LinkedNode;
-import org.apache.activemq.store.kahadb.disk.util.SequenceSet;
-import org.apache.activemq.util.*;
+import org.apache.activemq.store.kahadb.disk.util.LinkedNodeList;
+import org.apache.activemq.store.kahadb.disk.util.Sequence;
+import org.apache.activemq.util.ByteSequence;
+import org.apache.activemq.util.DataByteArrayInputStream;
+import org.apache.activemq.util.DataByteArrayOutputStream;
+import org.apache.activemq.util.IOHelper;
+import org.apache.activemq.util.RecoverableRandomAccessFile;
+import org.apache.activemq.util.ThreadPoolUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import org.apache.activemq.store.kahadb.disk.util.LinkedNodeList;
-import org.apache.activemq.store.kahadb.disk.util.SchedulerTimerTask;
-import org.apache.activemq.store.kahadb.disk.util.Sequence;
 
 /**
  * Manages DataFiles
- *
- *
  */
 public class Journal {
     public static final String CALLER_BUFFER_APPENDER = "org.apache.kahadb.journal.CALLER_BUFFER_APPENDER";
     public static final boolean callerBufferAppender = Boolean.parseBoolean(System.getProperty(CALLER_BUFFER_APPENDER, "false"));
 
-    private static final int MAX_BATCH_SIZE = 32*1024*1024;
+    private static final int PREALLOC_CHUNK_SIZE = 1024*1024;
 
     // ITEM_HEAD_SPACE = length + type+ reserved space + SOR
     public static final int RECORD_HEAD_SPACE = 4 + 1;
@@ -52,10 +74,16 @@ public class Journal {
     public static final byte BATCH_CONTROL_RECORD_TYPE = 2;
     // Batch Control Item holds a 4 byte size of the batch and a 8 byte checksum of the batch.
     public static final byte[] BATCH_CONTROL_RECORD_MAGIC = bytes("WRITE BATCH");
-    public static final int BATCH_CONTROL_RECORD_SIZE = RECORD_HEAD_SPACE+BATCH_CONTROL_RECORD_MAGIC.length+4+8;
+    public static final int BATCH_CONTROL_RECORD_SIZE = RECORD_HEAD_SPACE + BATCH_CONTROL_RECORD_MAGIC.length + 4 + 8;
     public static final byte[] BATCH_CONTROL_RECORD_HEADER = createBatchControlRecordHeader();
+    public static final byte[] EMPTY_BATCH_CONTROL_RECORD = createEmptyBatchControlRecordHeader();
+    public static final int EOF_INT = ByteBuffer.wrap(new byte[]{'-', 'q', 'M', 'a'}).getInt();
+    public static final byte EOF_EOT = '4';
+    public static final byte[] EOF_RECORD = createEofBatchAndLocationRecord();
 
-    // tackle corruption when checksum is disabled or corrupt with zeros, minimise data loss
+    private ScheduledExecutorService scheduler;
+
+    // tackle corruption when checksum is disabled or corrupt with zeros, minimize data loss
     public void corruptRecoveryLocation(Location recoveryPosition) throws IOException {
         DataFile dataFile = getDataFile(recoveryPosition);
         // with corruption on recovery we have no faith in the content - slip to the next batch record or eof
@@ -70,26 +98,37 @@ public class Journal {
             recoveryPosition.setSize(-1);
 
             dataFile.corruptedBlocks.add(sequence);
-
         } catch (IOException e) {
         } finally {
             accessorPool.closeDataFileAccessor(reader);
         }
     }
 
+    public DataFileAccessorPool getAccessorPool() {
+        return accessorPool;
+    }
+
     public enum PreallocationStrategy {
         SPARSE_FILE,
         OS_KERNEL_COPY,
-        ZEROS;
+        ZEROS,
+        CHUNKED_ZEROS;
     }
 
     public enum PreallocationScope {
-        ENTIRE_JOURNAL;
+        ENTIRE_JOURNAL,
+        ENTIRE_JOURNAL_ASYNC,
+        NONE;
+    }
+
+    public enum JournalDiskSyncStrategy {
+        ALWAYS,
+        PERIODIC,
+        NEVER;
     }
 
     private static byte[] createBatchControlRecordHeader() {
-        try {
-            DataByteArrayOutputStream os = new DataByteArrayOutputStream();
+        try (DataByteArrayOutputStream os = new DataByteArrayOutputStream();) {
             os.writeInt(BATCH_CONTROL_RECORD_SIZE);
             os.writeByte(BATCH_CONTROL_RECORD_TYPE);
             os.write(BATCH_CONTROL_RECORD_MAGIC);
@@ -101,13 +140,39 @@ public class Journal {
         }
     }
 
+    private static byte[] createEmptyBatchControlRecordHeader() {
+        try (DataByteArrayOutputStream os = new DataByteArrayOutputStream();) {
+            os.writeInt(BATCH_CONTROL_RECORD_SIZE);
+            os.writeByte(BATCH_CONTROL_RECORD_TYPE);
+            os.write(BATCH_CONTROL_RECORD_MAGIC);
+            os.writeInt(0);
+            os.writeLong(0l);
+            ByteSequence sequence = os.toByteSequence();
+            sequence.compact();
+            return sequence.getData();
+        } catch (IOException e) {
+            throw new RuntimeException("Could not create empty batch control record header.", e);
+        }
+    }
+
+    private static byte[] createEofBatchAndLocationRecord() {
+        try (DataByteArrayOutputStream os = new DataByteArrayOutputStream();) {
+            os.writeInt(EOF_INT);
+            os.writeByte(EOF_EOT);
+            ByteSequence sequence = os.toByteSequence();
+            sequence.compact();
+            return sequence.getData();
+        } catch (IOException e) {
+            throw new RuntimeException("Could not create eof header.", e);
+        }
+    }
+
     public static final String DEFAULT_DIRECTORY = ".";
     public static final String DEFAULT_ARCHIVE_DIRECTORY = "data-archive";
     public static final String DEFAULT_FILE_PREFIX = "db-";
     public static final String DEFAULT_FILE_SUFFIX = ".log";
     public static final int DEFAULT_MAX_FILE_LENGTH = 1024 * 1024 * 32;
     public static final int DEFAULT_CLEANUP_INTERVAL = 1000 * 30;
-    public static final int PREFERED_DIFF = 1024 * 512;
     public static final int DEFAULT_MAX_WRITE_BATCH_SIZE = 1024 * 1024 * 4;
 
     private static final Logger LOG = LoggerFactory.getLogger(Journal.class);
@@ -133,17 +198,22 @@ public class Journal {
     protected LinkedNodeList<DataFile> dataFiles = new LinkedNodeList<DataFile>();
 
     protected final AtomicReference<Location> lastAppendLocation = new AtomicReference<Location>();
-    protected Runnable cleanupTask;
+    protected ScheduledFuture cleanupTask;
     protected AtomicLong totalLength = new AtomicLong();
     protected boolean archiveDataLogs;
     private ReplicationTarget replicationTarget;
     protected boolean checksum;
     protected boolean checkForCorruptionOnStartup;
     protected boolean enableAsyncDiskSync = true;
-    private Timer timer;
+    private int nextDataFileId = 1;
+    private Object dataFileIdLock = new Object();
+    private final AtomicReference<DataFile> currentDataFile = new AtomicReference<>(null);
+    private volatile DataFile nextDataFile;
 
     protected PreallocationScope preallocationScope = PreallocationScope.ENTIRE_JOURNAL;
     protected PreallocationStrategy preallocationStrategy = PreallocationStrategy.SPARSE_FILE;
+    private File osKernelCopyTemplateFile = null;
+    protected JournalDiskSyncStrategy journalDiskSyncStrategy = JournalDiskSyncStrategy.ALWAYS;
 
     public interface DataFileRemovedListener {
         void fileRemoved(DataFile datafile);
@@ -163,6 +233,7 @@ public class Journal {
         appender = callerBufferAppender ? new CallerBufferingDataFileAppender(this) : new DataFileAppender(this);
 
         File[] files = directory.listFiles(new FilenameFilter() {
+            @Override
             public boolean accept(File dir, String n) {
                 return dir.equals(directory) && n.startsWith(filePrefix) && n.endsWith(fileSuffix);
             }
@@ -184,13 +255,15 @@ public class Journal {
 
             // Sort the list so that we can link the DataFiles together in the
             // right order.
-            List<DataFile> l = new ArrayList<DataFile>(fileMap.values());
+            LinkedList<DataFile> l = new LinkedList<>(fileMap.values());
             Collections.sort(l);
             for (DataFile df : l) {
                 if (df.getLength() == 0) {
                     // possibly the result of a previous failed write
                     LOG.info("ignoring zero length, partially initialised journal data file: " + df);
                     continue;
+                } else if (l.getLast().equals(df) && isUnusedPreallocated(df)) {
+                    continue;
                 }
                 dataFiles.addLast(df);
                 fileByFileMap.put(df.getFile(), df);
@@ -201,10 +274,30 @@ public class Journal {
             }
         }
 
-        getCurrentWriteFile();
+        if (preallocationScope != PreallocationScope.NONE && preallocationStrategy == PreallocationStrategy.OS_KERNEL_COPY) {
+            // create a template file that will be used to pre-allocate the journal files
+            if (osKernelCopyTemplateFile == null) {
+                osKernelCopyTemplateFile = createJournalTemplateFile();
+            }
+        }
 
-        if (preallocationStrategy != PreallocationStrategy.SPARSE_FILE && maxFileLength != DEFAULT_MAX_FILE_LENGTH) {
-            LOG.warn("You are using a preallocation strategy and journal maxFileLength which should be benchmarked accordingly to not introduce unexpected latencies.");
+        scheduler = Executors.newScheduledThreadPool(1, new ThreadFactory() {
+            @Override
+            public Thread newThread(Runnable r) {
+                Thread schedulerThread = new Thread(r);
+                schedulerThread.setName("ActiveMQ Journal Scheduled executor");
+                schedulerThread.setDaemon(true);
+                return schedulerThread;
+            }
+        });
+
+        // init current write file
+        if (dataFiles.isEmpty()) {
+            nextDataFileId = 1;
+            rotateWriteFile();
+        } else {
+            currentDataFile.set(dataFiles.getTail());
+            nextDataFileId = currentDataFile.get().dataFileId + 1;
         }
 
         if( lastAppendLocation.get()==null ) {
@@ -217,95 +310,124 @@ public class Journal {
             totalLength.addAndGet(lastAppendLocation.get().getOffset() - maxFileLength);
         }
 
-
-        cleanupTask = new Runnable() {
+        cleanupTask = scheduler.scheduleAtFixedRate(new Runnable() {
+            @Override
             public void run() {
                 cleanup();
             }
-        };
-        this.timer = new Timer("KahaDB Scheduler", true);
-        TimerTask task = new SchedulerTimerTask(cleanupTask);
-        this.timer.scheduleAtFixedRate(task, DEFAULT_CLEANUP_INTERVAL,DEFAULT_CLEANUP_INTERVAL);
+        }, DEFAULT_CLEANUP_INTERVAL, DEFAULT_CLEANUP_INTERVAL, TimeUnit.MILLISECONDS);
+
         long end = System.currentTimeMillis();
         LOG.trace("Startup took: "+(end-start)+" ms");
     }
 
-
     public void preallocateEntireJournalDataFile(RecoverableRandomAccessFile file) {
 
-        if (PreallocationScope.ENTIRE_JOURNAL == preallocationScope) {
+        if (PreallocationScope.NONE != preallocationScope) {
 
             if (PreallocationStrategy.OS_KERNEL_COPY == preallocationStrategy) {
                 doPreallocationKernelCopy(file);
-
-            }else if (PreallocationStrategy.ZEROS == preallocationStrategy) {
+            } else if (PreallocationStrategy.ZEROS == preallocationStrategy) {
                 doPreallocationZeros(file);
-            }
-            else {
+            } else if (PreallocationStrategy.CHUNKED_ZEROS == preallocationStrategy) {
+                doPreallocationChunkedZeros(file);
+            } else {
                 doPreallocationSparseFile(file);
             }
-        }else {
-            LOG.info("Using journal preallocation scope of batch allocation");
         }
     }
 
     private void doPreallocationSparseFile(RecoverableRandomAccessFile file) {
+        final ByteBuffer journalEof = ByteBuffer.wrap(EOF_RECORD);
         try {
-            file.seek(maxFileLength - 1);
-            file.write((byte)0x00);
+            FileChannel channel = file.getChannel();
+            channel.position(0);
+            channel.write(journalEof);
+            channel.position(maxFileLength - 5);
+            journalEof.rewind();
+            channel.write(journalEof);
+            channel.force(false);
+            channel.position(0);
+        } catch (ClosedByInterruptException ignored) {
+            LOG.trace("Could not preallocate journal file with sparse file", ignored);
         } catch (IOException e) {
-            LOG.error("Could not preallocate journal file with sparse file! Will continue without preallocation", e);
+            LOG.error("Could not preallocate journal file with sparse file", e);
         }
     }
 
     private void doPreallocationZeros(RecoverableRandomAccessFile file) {
         ByteBuffer buffer = ByteBuffer.allocate(maxFileLength);
-        for (int i = 0; i < maxFileLength; i++) {
-            buffer.put((byte) 0x00);
-        }
-        buffer.flip();
-
+        buffer.put(EOF_RECORD);
+        buffer.rewind();
         try {
             FileChannel channel = file.getChannel();
             channel.write(buffer);
             channel.force(false);
             channel.position(0);
+        } catch (ClosedByInterruptException ignored) {
+            LOG.trace("Could not preallocate journal file with zeros", ignored);
         } catch (IOException e) {
-            LOG.error("Could not preallocate journal file with zeros! Will continue without preallocation", e);
+            LOG.error("Could not preallocate journal file with zeros", e);
         }
     }
 
     private void doPreallocationKernelCopy(RecoverableRandomAccessFile file) {
-
-        // create a template file that will be used to pre-allocate the journal files
-        File templateFile = createJournalTemplateFile();
-
-        RandomAccessFile templateRaf = null;
         try {
-            templateRaf = new RandomAccessFile(templateFile, "rw");
-            templateRaf.setLength(maxFileLength);
-            templateRaf.getChannel().force(true);
+            RandomAccessFile templateRaf = new RandomAccessFile(osKernelCopyTemplateFile, "rw");
             templateRaf.getChannel().transferTo(0, getMaxFileLength(), file.getChannel());
             templateRaf.close();
-            templateFile.delete();
+        } catch (ClosedByInterruptException ignored) {
+            LOG.trace("Could not preallocate journal file with kernel copy", ignored);
         } catch (FileNotFoundException e) {
-            LOG.error("Could not find the template file on disk at " + templateFile.getAbsolutePath(), e);
+            LOG.error("Could not find the template file on disk at " + osKernelCopyTemplateFile.getAbsolutePath(), e);
         } catch (IOException e) {
-            LOG.error("Could not transfer the template file to journal, transferFile=" + templateFile.getAbsolutePath(), e);
+            LOG.error("Could not transfer the template file to journal, transferFile=" + osKernelCopyTemplateFile.getAbsolutePath(), e);
         }
     }
 
     private File createJournalTemplateFile() {
         String fileName = "db-log.template";
-        File rc  = new File(directory, fileName);
-        if (rc.exists()) {
-            System.out.println("deleting file because it already exists...");
-            rc.delete();
-
+        File rc = new File(directory, fileName);
+        try (RandomAccessFile templateRaf = new RandomAccessFile(rc, "rw");) {
+            templateRaf.getChannel().write(ByteBuffer.wrap(EOF_RECORD));
+            templateRaf.setLength(maxFileLength);
+            templateRaf.getChannel().force(true);
+        } catch (FileNotFoundException e) {
+            LOG.error("Could not find the template file on disk at " + osKernelCopyTemplateFile.getAbsolutePath(), e);
+        } catch (IOException e) {
+            LOG.error("Could not transfer the template file to journal, transferFile=" + osKernelCopyTemplateFile.getAbsolutePath(), e);
         }
         return rc;
     }
 
+    private void doPreallocationChunkedZeros(RecoverableRandomAccessFile file) {
+
+        ByteBuffer buffer = ByteBuffer.allocate(PREALLOC_CHUNK_SIZE);
+        buffer.put(EOF_RECORD);
+        buffer.rewind();
+
+        try {
+            FileChannel channel = file.getChannel();
+
+            int remLen = maxFileLength;
+            while (remLen > 0) {
+                if (remLen < buffer.remaining()) {
+                    buffer.limit(remLen);
+                }
+                int writeLen = channel.write(buffer);
+                remLen -= writeLen;
+                buffer.rewind();
+            }
+
+            channel.force(false);
+            channel.position(0);
+        } catch (ClosedByInterruptException ignored) {
+            LOG.trace("Could not preallocate journal file with zeros", ignored);
+        } catch (IOException e) {
+            LOG.error("Could not preallocate journal file with zeros! Will continue without preallocation", e);
+        }
+    }
+
     private static byte[] bytes(String string) {
         try {
             return string.getBytes("UTF-8");
@@ -314,6 +436,24 @@ public class Journal {
         }
     }
 
+    public boolean isUnusedPreallocated(DataFile dataFile) throws IOException {
+        int firstBatchRecordSize = -1;
+        if (preallocationScope == PreallocationScope.ENTIRE_JOURNAL_ASYNC) {
+            Location location = new Location();
+            location.setDataFileId(dataFile.getDataFileId());
+            location.setOffset(0);
+
+            DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
+            try {
+                firstBatchRecordSize = checkBatchRecord(reader, location.getOffset());
+            } catch (Exception ignored) {
+            } finally {
+                accessorPool.closeDataFileAccessor(reader);
+            }
+        }
+        return firstBatchRecordSize == 0;
+    }
+
     protected Location recoveryCheck(DataFile dataFile) throws IOException {
         Location location = new Location();
         location.setDataFileId(dataFile.getDataFileId());
@@ -321,16 +461,21 @@ public class Journal {
 
         DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
         try {
-            while( true ) {
+            while (true) {
                 int size = checkBatchRecord(reader, location.getOffset());
-                if ( size>=0 && location.getOffset()+BATCH_CONTROL_RECORD_SIZE+size <= dataFile.getLength()) {
-                    location.setOffset(location.getOffset()+BATCH_CONTROL_RECORD_SIZE+size);
+                if (size >= 0 && location.getOffset() + BATCH_CONTROL_RECORD_SIZE + size <= dataFile.getLength()) {
+                    if (size == 0) {
+                        // eof batch record
+                        break;
+                    }
+                    location.setOffset(location.getOffset() + BATCH_CONTROL_RECORD_SIZE + size);
                 } else {
 
-                    // Perhaps it's just some corruption... scan through the file to find the next valid batch record.  We
+                    // Perhaps it's just some corruption... scan through the
+                    // file to find the next valid batch record. We
                     // may have subsequent valid batch records.
-                    int nextOffset = findNextBatchRecord(reader, location.getOffset()+1);
-                    if( nextOffset >=0 ) {
+                    int nextOffset = findNextBatchRecord(reader, location.getOffset() + 1);
+                    if (nextOffset >= 0) {
                         Sequence sequence = new Sequence(location.getOffset(), nextOffset - 1);
                         LOG.warn("Corrupt journal records found in '" + dataFile.getFile() + "' between offsets: " + sequence);
                         dataFile.corruptedBlocks.add(sequence);
@@ -352,9 +497,9 @@ public class Journal {
             totalLength.addAndGet(dataFile.getLength() - existingLen);
         }
 
-        if( !dataFile.corruptedBlocks.isEmpty() ) {
+        if (!dataFile.corruptedBlocks.isEmpty()) {
             // Is the end of the data file corrupted?
-            if( dataFile.corruptedBlocks.getTail().getLast()+1 == location.getOffset() ) {
+            if (dataFile.corruptedBlocks.getTail().getLast() + 1 == location.getOffset()) {
                 dataFile.setLength((int) dataFile.corruptedBlocks.removeLastSequence().getFirst());
             }
         }
@@ -368,66 +513,71 @@ public class Journal {
         ByteSequence bs = new ByteSequence(data, 0, reader.read(offset, data));
 
         int pos = 0;
-        while( true ) {
+        while (true) {
             pos = bs.indexOf(header, pos);
-            if( pos >= 0 ) {
-                return offset+pos;
+            if (pos >= 0) {
+                return offset + pos;
             } else {
                 // need to load the next data chunck in..
-                if( bs.length != data.length ) {
+                if (bs.length != data.length) {
                     // If we had a short read then we were at EOF
                     return -1;
                 }
-                offset += bs.length-BATCH_CONTROL_RECORD_HEADER.length;
+                offset += bs.length - BATCH_CONTROL_RECORD_HEADER.length;
                 bs = new ByteSequence(data, 0, reader.read(offset, data));
-                pos=0;
+                pos = 0;
             }
         }
     }
 
-
     public int checkBatchRecord(DataFileAccessor reader, int offset) throws IOException {
         byte controlRecord[] = new byte[BATCH_CONTROL_RECORD_SIZE];
-        DataByteArrayInputStream controlIs = new DataByteArrayInputStream(controlRecord);
 
-        reader.readFully(offset, controlRecord);
+        try (DataByteArrayInputStream controlIs = new DataByteArrayInputStream(controlRecord);) {
 
-        // Assert that it's  a batch record.
-        for( int i=0; i < BATCH_CONTROL_RECORD_HEADER.length; i++ ) {
-            if( controlIs.readByte() != BATCH_CONTROL_RECORD_HEADER[i] ) {
-                return -1;
-            }
-        }
+            reader.readFully(offset, controlRecord);
 
-        int size = controlIs.readInt();
-        if( size > MAX_BATCH_SIZE ) {
-            return -1;
-        }
-
-        if( isChecksum() ) {
-
-            long expectedChecksum = controlIs.readLong();
-            if( expectedChecksum == 0 ) {
-                // Checksuming was not enabled when the record was stored.
-                // we can't validate the record :(
-                return size;
+            // check for journal eof
+            if (Arrays.equals(EOF_RECORD, Arrays.copyOfRange(controlRecord, 0, EOF_RECORD.length))) {
+                // eof batch
+                return 0;
             }
 
-            byte data[] = new byte[size];
-            reader.readFully(offset+BATCH_CONTROL_RECORD_SIZE, data);
+            // Assert that it's a batch record.
+            for (int i = 0; i < BATCH_CONTROL_RECORD_HEADER.length; i++) {
+                if (controlIs.readByte() != BATCH_CONTROL_RECORD_HEADER[i]) {
+                    return -1;
+                }
+            }
 
-            Checksum checksum = new Adler32();
-            checksum.update(data, 0, data.length);
-
-            if( expectedChecksum!=checksum.getValue() ) {
+            int size = controlIs.readInt();
+            if (size < 0 || size > Integer.MAX_VALUE - (BATCH_CONTROL_RECORD_SIZE + EOF_RECORD.length)) {
                 return -1;
             }
 
+            if (isChecksum()) {
+
+                long expectedChecksum = controlIs.readLong();
+                if (expectedChecksum == 0) {
+                    // Checksuming was not enabled when the record was stored.
+                    // we can't validate the record :(
+                    return size;
+                }
+
+                byte data[] = new byte[size];
+                reader.readFully(offset + BATCH_CONTROL_RECORD_SIZE, data);
+
+                Checksum checksum = new Adler32();
+                checksum.update(data, 0, data.length);
+
+                if (expectedChecksum != checksum.getValue()) {
+                    return -1;
+                }
+            }
+            return size;
         }
-        return size;
     }
 
-
     void addToTotalLength(int size) {
         totalLength.addAndGet(size);
     }
@@ -436,32 +586,81 @@ public class Journal {
         return totalLength.get();
     }
 
-    synchronized DataFile getCurrentWriteFile() throws IOException {
-        if (dataFiles.isEmpty()) {
-            rotateWriteFile();
+    private void rotateWriteFile() throws IOException {
+       synchronized (dataFileIdLock) {
+            DataFile dataFile = nextDataFile;
+            if (dataFile == null) {
+                dataFile = newDataFile();
+            }
+            synchronized (currentDataFile) {
+                fileMap.put(dataFile.getDataFileId(), dataFile);
+                fileByFileMap.put(dataFile.getFile(), dataFile);
+                dataFiles.addLast(dataFile);
+                currentDataFile.set(dataFile);
+            }
+            nextDataFile = null;
+        }
+        if (PreallocationScope.ENTIRE_JOURNAL_ASYNC == preallocationScope) {
+            preAllocateNextDataFileFuture = scheduler.submit(preAllocateNextDataFileTask);
         }
-        return dataFiles.getTail();
     }
 
-    synchronized DataFile rotateWriteFile() {
-        int nextNum = !dataFiles.isEmpty() ? dataFiles.getTail().getDataFileId().intValue() + 1 : 1;
+    private Runnable preAllocateNextDataFileTask = new Runnable() {
+        @Override
+        public void run() {
+            if (nextDataFile == null) {
+                synchronized (dataFileIdLock){
+                    try {
+                        nextDataFile = newDataFile();
+                    } catch (IOException e) {
+                        LOG.warn("Failed to proactively allocate data file", e);
+                    }
+                }
+            }
+        }
+    };
+
+    private volatile Future preAllocateNextDataFileFuture;
+
+    private DataFile newDataFile() throws IOException {
+        int nextNum = nextDataFileId++;
         File file = getFile(nextNum);
         DataFile nextWriteFile = new DataFile(file, nextNum);
-        fileMap.put(nextWriteFile.getDataFileId(), nextWriteFile);
-        fileByFileMap.put(file, nextWriteFile);
-        dataFiles.addLast(nextWriteFile);
+        preallocateEntireJournalDataFile(nextWriteFile.appendRandomAccessFile());
         return nextWriteFile;
     }
 
+
+    public DataFile reserveDataFile() {
+        synchronized (dataFileIdLock) {
+            int nextNum = nextDataFileId++;
+            File file = getFile(nextNum);
+            DataFile reservedDataFile = new DataFile(file, nextNum);
+            synchronized (currentDataFile) {
+                fileMap.put(reservedDataFile.getDataFileId(), reservedDataFile);
+                fileByFileMap.put(file, reservedDataFile);
+                if (dataFiles.isEmpty()) {
+                    dataFiles.addLast(reservedDataFile);
+                } else {
+                    dataFiles.getTail().linkBefore(reservedDataFile);
+                }
+            }
+            return reservedDataFile;
+        }
+    }
+
     public File getFile(int nextNum) {
         String fileName = filePrefix + nextNum + fileSuffix;
         File file = new File(directory, fileName);
         return file;
     }
 
-    synchronized DataFile getDataFile(Location item) throws IOException {
+    DataFile getDataFile(Location item) throws IOException {
         Integer key = Integer.valueOf(item.getDataFileId());
-        DataFile dataFile = fileMap.get(key);
+        DataFile dataFile = null;
+        synchronized (currentDataFile) {
+            dataFile = fileMap.get(key);
+        }
         if (dataFile == null) {
             LOG.error("Looking for key " + key + " but not found in fileMap: " + fileMap);
             throw new IOException("Could not locate data file " + getFile(item.getDataFileId()));
@@ -469,33 +668,21 @@ public class Journal {
         return dataFile;
     }
 
-    synchronized File getFile(Location item) throws IOException {
-        Integer key = Integer.valueOf(item.getDataFileId());
-        DataFile dataFile = fileMap.get(key);
-        if (dataFile == null) {
-            LOG.error("Looking for key " + key + " but not found in fileMap: " + fileMap);
-            throw new IOException("Could not locate data file " + getFile(item.getDataFileId()));
-        }
-        return dataFile.getFile();
-    }
-
-    private DataFile getNextDataFile(DataFile dataFile) {
-        return dataFile.getNext();
-    }
-
     public void close() throws IOException {
         synchronized (this) {
             if (!started) {
                 return;
             }
-            if (this.timer != null) {
-                this.timer.cancel();
+            cleanupTask.cancel(true);
+            if (preAllocateNextDataFileFuture != null) {
+                preAllocateNextDataFileFuture.cancel(true);
             }
+            ThreadPoolUtils.shutdownGraceful(scheduler, 4000);
             accessorPool.close();
         }
         // the appender can be calling back to to the journal blocking a close AMQ-5620
         appender.close();
-        synchronized (this) {
+        synchronized (currentDataFile) {
             fileMap.clear();
             fileByFileMap.clear();
             dataFiles.clear();
@@ -504,7 +691,7 @@ public class Journal {
         }
     }
 
-    protected synchronized void cleanup() {
+    public synchronized void cleanup() {
         if (accessorPool != null) {
             accessorPool.disposeUnused();
         }
@@ -521,37 +708,53 @@ public class Journal {
             DataFile dataFile = i.next();
             result &= dataFile.delete();
         }
-        totalLength.set(0);
-        fileMap.clear();
-        fileByFileMap.clear();
-        lastAppendLocation.set(null);
-        dataFiles = new LinkedNodeList<DataFile>();
 
+        if (preAllocateNextDataFileFuture != null) {
+            preAllocateNextDataFileFuture.cancel(true);
+        }
+        synchronized (dataFileIdLock) {
+            if (nextDataFile != null) {
+                nextDataFile.delete();
+                nextDataFile = null;
+            }
+        }
+
+        totalLength.set(0);
+        synchronized (currentDataFile) {
+            fileMap.clear();
+            fileByFileMap.clear();
+            lastAppendLocation.set(null);
+            dataFiles = new LinkedNodeList<DataFile>();
+        }
         // reopen open file handles...
         accessorPool = new DataFileAccessorPool(this);
         appender = new DataFileAppender(this);
         return result;
     }
 
-    public synchronized void removeDataFiles(Set<Integer> files) throws IOException {
+    public void removeDataFiles(Set<Integer> files) throws IOException {
         for (Integer key : files) {
             // Can't remove the data file (or subsequent files) that is currently being written to.
-            if( key >= lastAppendLocation.get().getDataFileId() ) {
+            if (key >= lastAppendLocation.get().getDataFileId()) {
                 continue;
             }
-            DataFile dataFile = fileMap.get(key);
-            if( dataFile!=null ) {
+            DataFile dataFile = null;
+            synchronized (currentDataFile) {
+                dataFile = fileMap.remove(key);
+                if (dataFile != null) {
+                    fileByFileMap.remove(dataFile.getFile());
+                    dataFile.unlink();
+                }
+            }
+            if (dataFile != null) {
                 forceRemoveDataFile(dataFile);
             }
         }
     }
 
-    private synchronized void forceRemoveDataFile(DataFile dataFile) throws IOException {
+    private void forceRemoveDataFile(DataFile dataFile) throws IOException {
         accessorPool.disposeDataFileAccessors(dataFile);
-        fileByFileMap.remove(dataFile.getFile());
-        fileMap.remove(dataFile.getDataFileId());
         totalLength.addAndGet(-dataFile.getLength());
-        dataFile.unlink();
         if (archiveDataLogs) {
             File directoryArchive = getDirectoryArchive();
             if (directoryArchive.exists()) {
@@ -569,7 +772,7 @@ public class Journal {
             LOG.debug("Successfully moved data file");
         } else {
             LOG.debug("Deleting data file: {}", dataFile);
-            if ( dataFile.delete() ) {
+            if (dataFile.delete()) {
                 LOG.debug("Discarded data file: {}", dataFile);
             } else {
                 LOG.warn("Failed to discard data file : {}", dataFile.getFile());
@@ -599,14 +802,16 @@ public class Journal {
         return directory.toString();
     }
 
-    public synchronized Location getNextLocation(Location location) throws IOException, IllegalStateException {
-
+    public Location getNextLocation(Location location) throws IOException, IllegalStateException {
         Location cur = null;
         while (true) {
             if (cur == null) {
                 if (location == null) {
-                    DataFile head = dataFiles.getHead();
-                    if( head == null ) {
+                    DataFile head = null;
+                    synchronized (currentDataFile) {
+                        head = dataFiles.getHead();
+                    }
+                    if (head == null) {
                         return null;
                     }
                     cur = new Location();
@@ -629,7 +834,9 @@ public class Journal {
 
             // Did it go into the next file??
             if (dataFile.getLength() <= cur.getOffset()) {
-                dataFile = getNextDataFile(dataFile);
+                synchronized (currentDataFile) {
+                    dataFile = dataFile.getNext();
+                }
                 if (dataFile == null) {
                     return null;
                 } else {
@@ -650,9 +857,14 @@ public class Journal {
             if (corruptedRange != null) {
                 // skip corruption
                 cur.setSize((int) corruptedRange.range());
-            } else if (cur.getType() == 0) {
+            } else if (cur.getSize() == EOF_INT && cur.getType() == EOF_EOT ||
+                    (cur.getType() == 0 && cur.getSize() == 0)) {
                 // eof - jump to next datafile
-                cur.setOffset(maxFileLength);
+                // EOF_INT and EOF_EOT replace 0,0 - we need to react to both for
+                // replay of existing journals
+                // possibly journal is larger than maxFileLength after config change
+                cur.setSize(EOF_RECORD.length);
+                cur.setOffset(Math.max(maxFileLength, dataFile.getLength()));
             } else if (cur.getType() == USER_RECORD_TYPE) {
                 // Only return user records.
                 return cur;
@@ -660,7 +872,7 @@ public class Journal {
         }
     }
 
-    public synchronized ByteSequence read(Location location) throws IOException, IllegalStateException {
+    public ByteSequence read(Location location) throws IOException, IllegalStateException {
         DataFile dataFile = getDataFile(location);
         DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
         ByteSequence rc = null;
@@ -758,10 +970,36 @@ public class Journal {
         this.archiveDataLogs = archiveDataLogs;
     }
 
-    synchronized public Integer getCurrentDataFileId() {
-        if (dataFiles.isEmpty())
-            return null;
-        return dataFiles.getTail().getDataFileId();
+    public DataFile getDataFileById(int dataFileId) {
+        synchronized (currentDataFile) {
+            return fileMap.get(Integer.valueOf(dataFileId));
+        }
+    }
+
+    public DataFile getCurrentDataFile(int capacity) throws IOException {
+        //First just acquire the currentDataFile lock and return if no rotation needed
+        synchronized (currentDataFile) {
+            if (currentDataFile.get().getLength() + capacity < maxFileLength) {
+                return currentDataFile.get();
+            }
+        }
+
+        //AMQ-6545 - if rotation needed, acquire dataFileIdLock first to prevent deadlocks
+        //then re-check if rotation is needed
+        synchronized (dataFileIdLock) {
+            synchronized (currentDataFile) {
+                if (currentDataFile.get().getLength() + capacity >= maxFileLength) {
+                    rotateWriteFile();
+                }
+                return currentDataFile.get();
+            }
+        }
+    }
+
+    public Integer getCurrentDataFileId() {
+        synchronized (currentDataFile) {
+            return currentDataFile.get().getDataFileId();
+        }
     }
 
     /**
@@ -770,11 +1008,15 @@ public class Journal {
      * @return files currently being used
      */
     public Set<File> getFiles() {
-        return fileByFileMap.keySet();
+        synchronized (currentDataFile) {
+            return fileByFileMap.keySet();
+        }
     }
 
-    public synchronized Map<Integer, DataFile> getFileMap() {
-        return new TreeMap<Integer, DataFile>(fileMap);
+    public Map<Integer, DataFile> getFileMap() {
+        synchronized (currentDataFile) {
+            return new TreeMap<Integer, DataFile>(fileMap);
+        }
     }
 
     public long getDiskSize() {
@@ -784,6 +1026,7 @@ public class Journal {
     public void setReplicationTarget(ReplicationTarget replicationTarget) {
         this.replicationTarget = replicationTarget;
     }
+
     public ReplicationTarget getReplicationTarget() {
         return replicationTarget;
     }
@@ -832,6 +1075,18 @@ public class Journal {
         return enableAsyncDiskSync;
     }
 
+    public JournalDiskSyncStrategy getJournalDiskSyncStrategy() {
+        return journalDiskSyncStrategy;
+    }
+
+    public void setJournalDiskSyncStrategy(JournalDiskSyncStrategy journalDiskSyncStrategy) {
+        this.journalDiskSyncStrategy = journalDiskSyncStrategy;
+    }
+
+    public boolean isJournalDiskSyncPeriodic() {
+        return JournalDiskSyncStrategy.PERIODIC.equals(journalDiskSyncStrategy);
+    }
+
     public void setDataFileRemovedListener(DataFileRemovedListener dataFileRemovedListener) {
         this.dataFileRemovedListener = dataFileRemovedListener;
     }
@@ -869,10 +1124,12 @@ public class Journal {
             hash = (int)(file ^ offset);
         }
 
+        @Override
         public int hashCode() {
             return hash;
         }
 
+        @Override
         public boolean equals(Object obj) {
             if (obj instanceof WriteKey) {
                 WriteKey di = (WriteKey)obj;
